#FILE: align_learn.pl
#AUTH: Paul Stothard (paul.stothard@gmail.com)
#DATE: March 5, 2010
#VERS: 1.0


use strict;
use warnings;

#These simple modules were written for this script.
use ARFF::Sequence;
use ARFF::Attribute;
use ARFF::Data;
use ARFF::Relation;
use Getopt::Long;

my $inputFile;
my $outputFile;
my $relationTitle;
my $columns;

Getopt::Long::Configure('bundling');
GetOptions(
    'i|input_file=s'     => \$inputFile,
    'o|output_file=s'    => \$outputFile,
    't|relation_title=s' => \$relationTitle,
    'c|columns=s'        => \$columns
);

if ( ( !defined($inputFile) ) || ( !defined($outputFile) ) ) {
    print_usage();
    exit(1);
}

if ( !defined($relationTitle) ) {
    $relationTitle = "Untitled";
}

#create a Sequence object for each sequence in the input file
my @arrayOfSequences = ();
my $sequenceTitle;
my $sequenceClass;

open( ALIGNFILE, $inputFile ) or die("Cannot open file : $!");

local $/ = ">";

while ( my $sequenceEntry = <ALIGNFILE> ) {

    if ( $sequenceEntry eq ">" ) {
        next;
    }

    #sequence title CLASS: high
    if ( $sequenceEntry =~ m/^([^\n]+)class\:\s*([^\n]+)\n/i ) {
        $sequenceTitle = $1;
        $sequenceClass = $2;
    }
    else {
        die("A sequence title could not properly be read for this entry:\n $sequenceEntry."
        );
    }

    #remove title
    $sequenceEntry =~ s/^[^\n]+//;

    #remove white space and >
    $sequenceEntry =~ s/[\s\>]//g;

    #convert white-space in class to underscore
    $sequenceClass =~ s/\s+/\-/g;

    #create sequenceObject
    my $sequenceObject = new ARFF::Sequence;
    $sequenceObject->setClass($sequenceClass);
    $sequenceObject->setSequence($sequenceEntry);

    push( @arrayOfSequences, $sequenceObject );
}

close(ALIGNFILE) or die("Cannot close file : $!");

#Do a quick check of the sequences to confirm that they contain the same number of characters
my $length;
foreach (@arrayOfSequences) {
    if ( defined($length) ) {
        if ( $length != $_->getLength() ) {
            die("The sequences in the alignment must all contain the same number of characters"
            );
        }
    }
    else {
        $length = $_->getLength();
    }
}

#convert the column text into an array of digits and confirm that each position is acceptable.
my @arrayOfPositions = ();
if ( defined($columns) ) {
    while ( $columns =~ m/(\d+)/g ) {
        if ( ( $1 < 1 ) || ( $1 > $length ) ) {
            die("The position values for the entered sequences must be between 1 and $length"
            );
        }

        push( @arrayOfPositions, $1 );
    }
}

#use all positions
else {
    for ( my $i = 1; $i <= $length; $i++ ) {
        push( @arrayOfPositions, $i );
    }
}

#create a Relation object
my $relation = new ARFF::Relation;
$relation->setTitle($relationTitle);

#for each position, create an Attribute object and add it to the relation.
foreach (@arrayOfPositions) {
    my $attribute = new ARFF::Attribute;
    my $position  = $_;
    $attribute->setTitle( "position_" . $position );

    foreach (@arrayOfSequences) {
        $attribute->addValue( $_->getResidue($position) );
    }
    $relation->addAttribute($attribute);
}

#create an Attribute object for the sequence classes.
my $attribute = new ARFF::Attribute;
$attribute->setTitle("class");
foreach (@arrayOfSequences) {
    $attribute->addValue( $_->getClass() );
}
$relation->addAttribute($attribute);

#now create the data entry for each sequence
foreach (@arrayOfSequences) {
    my $data     = new ARFF::Data;
    my $sequence = $_;
    foreach (@arrayOfPositions) {
        $data->addValue( $sequence->getResidue($_) );
    }

    #add the class as the last entry
    $data->addValue( $sequence->getClass() );
    $relation->addData($data);
}

#now write the relation to file
open( OUTFILE, ">" . $outputFile ) or die("Cannot open file for output: $!");
print( OUTFILE $relation->toString() );
close(OUTFILE) or die("Cannot close file for output: $!");

sub print_usage {
    print <<BLOCK;
USAGE:
perl align_learn.pl [-arguments]

-i [FILE]        : multiple alignment file (Required).
-o [FILE]        : the ARFF file to create (Required).
-c [LIST]        : comma-separated list of alignment column numbers
                   (Optional; default is to use all columns).
-t [STRING]      : title for the relation (Optional).

perl align_learn.pl -i test_input/test_in.txt -o test_output/out.arff \
-t 'test relation'
BLOCK
}
